/*
 * HF synthesizer
 * base on Si5351x2
 * 9 - bands
 *
 * Main.c
 *
 * Prject modified: 18.11.2022
 * Author: R9OFG (ex R0AEK)
 * Version - 2.0 beta
 *
 * radio0aek@gmail.com
 * https://r0aek.me
 */ 

/*
   EEPROM

00...01		
02			  									1-
03			  								1-
04			  BFO_LSB								4-
08			  XTALL_1								4-
12			     160m	4-
16			     80m	4-
20			     40m	4-
24			     30m	4-
28			     20m	4-
32			     17m	4-
36			     15m	4-
40			     12m	4-
44			     10m	4-
48			  ATT/PRE 					9-
57			   					9-
66			    CLK0 Si5351-1				1-
67			  BFO_USB								4-
71			  XTALL_2								4-
75			    CLK0 Si5351-2				1-
76			  BFO_CWL								4-
80			  BFO_CWU								4-
84			   					1-
85			    VFO/BFO_xxx  TX		1-

*/

#include "Main.h"

/* */
uint8_t  data_8_EEPROM;										//    EEPROM

unsigned char butBuf				= BUT_NULL;				// 
unsigned char data_ADC_1			= 0;					//   ADC1
unsigned char compare				= 0;					//   

int old_S_metr_Value				= 0;					//   S-

unsigned int but_Count_Pressed		= 0;					//    
uint8_t but_Last_Pres				= 0;					//   
uint8_t but_ATT_PRE					= 0;					//   ATT  PRE (0 - off, 1 - ATT, 2 - PRE)
uint8_t but_MODE					= 0;					//  (0 - LSB, 1 - USB, 2 - CW)
uint8_t butPress_F					= 0;					//   (0 - , 1 - )
uint8_t ptt_cat_ON_F				= 0;					//  PTT  CAT
uint8_t butPressFUNC				= 0;					//   FUNC (0 - , 1 - )
uint8_t modeSetup					= 0;					//   

volatile uint8_t next_state;								//     
volatile uint8_t prev_state;								//-
volatile uint8_t up_state;									//-
volatile uint8_t down_state;								//-

uint8_t ABCD_Code					= 0x00;					//  ABCD       ATT_PRE/CW/PTT

uint8_t func_Mode					= FUNC_MODE_NORMAL;		//    
uint8_t mode_set					= 0;					//      (0 - off, 1 - on)
char TempStr[10]					= "";					//      
unsigned char band_Num				= BAND_40m;				//   
															/*  */
const uint32_t band_F_BUFF[9]		= {1840000,3600000,7100000,10130000,14101000,18111000,21151000,24931000,28225000};
															/*    */
uint32_t band_last_F_BUFF[9]		= {1840000,3600000,7100000,10130000,14101000,18111000,21151000,24931000,28225000};
															/*  */	
char* band_Label[9]					= {"160m","80m","40m","30m","20m","17m","15m","12m","10m"};
															/*       */
const int band_last_F_data_addr[9]	= {12,16,20,24,28,32,36,40,44};
															/*     Hz*/
const uint32_t step_Value[7]		= {1,10,100,500,1000,10000,100000};

uint32_t freq_BFO_LSB				= FREQ_LSB_DEF;			//  c  BFO SSB LOW    Hz
uint32_t freq_BFO_USB				= FREQ_USB_DEF;			//  c  BFO SSB HIGH    Hz
uint32_t freq_BFO_CWL				= FREQ_CWL_DEF;			//  c  BFO CW LOW    Hz
uint32_t freq_BFO_CWU				= FREQ_CWU_DEF;			//  c  BFO CW HIGH    Hz
uint32_t freq_VFO					= 7100000;				//  c  VFO    Hz
uint32_t xtall_FREQ_1			    = FREQ_XTALL_1_DEF;		//      Si5351 VFO
uint32_t xtall_FREQ_2			    = FREQ_XTALL_2_DEF;		//      Si5351 BFO
uint32_t freq_si					= 0;					//   c  Si5351  Hz
uint8_t  att_pre[9]					= {0,0,0,0,0,0,0,0,0};	// ATT/PRE   (0 - off, 1 - att, 2 - pre)
uint8_t  band_use[9]				= {1,1,1,1,1,1,1,1,1};	//   (0 - off, 1 - on)

uint64_t set_Step					= 0;					//     
uint16_t timerCountSaveBandData		= 0;					//      (freq, att/pre)
uint8_t  set_Step_index				= 0;					//     
uint8_t  set_Step_F					= 0;					//     
uint8_t  bandDataNum				= 0;					//    
uint8_t	 siDSC_1					= 0;					//       CLK0 Si5351 VFO (0 - 2mA, 1 - 4mA, 2 - 6mA, 3 - 8mA)
uint8_t	 siDSC_2					= 0;					//       CLK0 Si5351 BFO (0 - 2mA, 1 - 4mA, 2 - 6mA, 3 - 8mA)
uint8_t	 var_Conversion				= 0;					//       (0 - , 1 -  )
uint8_t  var_Swap_F_TX				= 0;					//        TX (0 - off, 1 - on)

// **********************************************************************************************************************/

void Read_Setup_EEPROM(void)								//   EEPROM
{
	/*   Hz*/
	data_8_EEPROM = eeprom_read_byte((uint8_t*)0x02);
	if ((data_8_EEPROM >= 0)&&(data_8_EEPROM <= 6))			//     step_Value
	set_Step = step_Value[data_8_EEPROM];				//
	else //
	{
		set_Step = step_Value[3];							// 100 Hz
		eeprom_write_byte((uint8_t*)0x02, step_Value[3]);	//   EEPROM       step_Value
	}
	
	/*    */
	data_8_EEPROM = eeprom_read_byte((uint8_t*)0x03);
	if ((data_8_EEPROM >= 1)&&(data_8_EEPROM <= 9))			//      BAND-of
	band_Num = data_8_EEPROM;							//
	else //
	eeprom_write_byte((uint8_t*)0x03,band_Num);			//     EEPROM
	
	/*  Freq_BFO_LSB  Hz*/
	freq_BFO_LSB = eeprom_read_float((float*)0x04);
	if (freq_BFO_LSB == 0)									//   = 0
	{
		freq_BFO_LSB = FREQ_LSB_DEF;
		eeprom_write_float((float*)0x04,freq_BFO_LSB);		//  EEPROM    
	}
	
	/*  Freq_BFO_USB  Hz*/
	freq_BFO_USB = eeprom_read_float((float*)67);
	if (freq_BFO_USB == 0)									//   = 0
	{
		freq_BFO_USB = FREQ_USB_DEF;
		eeprom_write_float((float*)67,freq_BFO_USB);		//  EEPROM    
	}
	
	/*  Freq_BFO_CWL  Hz*/
	freq_BFO_CWL = eeprom_read_float((float*)76);
	if (freq_BFO_CWL == 0)									//   = 0
	{
		freq_BFO_CWL = FREQ_CWL_DEF;
		eeprom_write_float((float*)76,freq_BFO_CWL);		//  EEPROM    
	}
	
	/*  Freq_BFO_CWU  Hz*/
	freq_BFO_CWU = eeprom_read_float((float*)80);
	if (freq_BFO_CWU == 0)									//   = 0
	{
		freq_BFO_CWU = FREQ_CWU_DEF;
		eeprom_write_float((float*)80,freq_BFO_CWU);		//  EEPROM    
	}
	
	/* XTALL_1*/
	xtall_FREQ_1 = eeprom_read_float((float*)0x08);
	if (xtall_FREQ_1 == 0)									//   = 0
	{
		xtall_FREQ_1 = FREQ_XTALL_1_DEF;
		eeprom_write_float((float*)0x08,xtall_FREQ_1);		//  EEPROM   BFO  
	}
	
	/* XTALL_2*/
	xtall_FREQ_2 = eeprom_read_float((float*)71);
	if (xtall_FREQ_2 == 0)									//   = 0
	{
		xtall_FREQ_2 = FREQ_XTALL_2_DEF;
		eeprom_write_float((float*)71,xtall_FREQ_2);		//  EEPROM   BFO  
	}
	
	/*    */
	for (int i=0;i<9;i++)
	{
		band_last_F_BUFF[i] = eeprom_read_float((float*)band_last_F_data_addr[i]);
		
		//       ,    -     EEPROM   
		if (band_last_F_BUFF[i] == 0)
		{
			band_last_F_BUFF[i] = band_F_BUFF[i];
			eeprom_write_float((float*)band_last_F_data_addr[i],band_F_BUFF[i]);
		}
	}
	
	/*  ATT/PRE */
	for (int i=0;i<9;i++)
	{
		att_pre[i] = eeprom_read_byte((uint8_t*)48 + i);
		//    0,1,2    -     EEPROM    - 0
		if ((att_pre[i] != 0)&&(att_pre[i] > 2))
		{
			att_pre[i] = 0;
			eeprom_write_byte((uint8_t*)48 + i, 0);
		}
	}
	
	/*   */
	for (int i=0;i<9;i++)
	{
		band_use[i] = eeprom_read_byte((uint8_t*)57 + i);
		//    0,1    -     EEPROM    - 1
		if ((band_use[i] != 0)&&(band_use[i] > 1))
		{
			band_use[i] = 1;
			eeprom_write_byte((uint8_t*)57 + i, 1);
		}
	}
	
	/*    CLK0 Si5351-1*/
	siDSC_1 = eeprom_read_byte((uint8_t*)66);
	//    0,1,2,3    -      EEPROM    - 3
	if ((siDSC_1 != 0)&&(siDSC_1 > 3))
	{
		siDSC_1 = 3;
		eeprom_write_byte((uint8_t*)66, siDSC_1);
	}
	
	/*    CLK0 Si5351-2*/
	siDSC_2 = eeprom_read_byte((uint8_t*)75);
	//    0,1,2,3    -      EEPROM    - 3
	if ((siDSC_2 != 0)&&(siDSC_2 > 3))
	{
		siDSC_2 = 3;
		eeprom_write_byte((uint8_t*)75, siDSC_2);
	}
	
	/*   */
	var_Conversion = eeprom_read_byte((uint8_t*)84);
	//    0  1,    -      EEPROM    - 0
	if ((var_Conversion != 0)&&(var_Conversion > 1))
	{
		var_Conversion = 0;
		eeprom_write_byte((uint8_t*)84, var_Conversion);
	}
	
	/*    VFO/BFO_xxx  TX*/
	var_Swap_F_TX = eeprom_read_byte((uint8_t*)85);
	//    0  1,    -      EEPROM    - 0
	if ((var_Swap_F_TX != 0)&&(var_Swap_F_TX > 1))
	{
		var_Swap_F_TX = 0;
		eeprom_write_byte((uint8_t*)85, var_Swap_F_TX);
	}
}

void SCL_1_line_Control(int value)							//  SCL1
{
	switch(value)
	{
		/*
		  :  1 -    "",  0 -  
		*/
		case 0: //off SCL1 line
			mode_set = 0;
			PORTC |= (1<<SW_I2C);
			_delay_ms(2);
			break;
		case 1: //on SCL1 line
			mode_set = 1;
			PORTC &= ~(1<<SW_I2C);
			_delay_ms(2);
			break;
	}
}

void Calc_Out_Freq_Si5351(void)								// VFO
{
	if (var_Conversion == 1)
	{
		//  -  
		switch(but_MODE)
		{
			case 0: //LSB
				freq_si = freq_VFO + freq_BFO_LSB;
				break;
			case 1: //USB
				freq_si = freq_VFO + freq_BFO_USB;
				break;
			case 2: //CWL
				freq_si = freq_VFO + freq_BFO_CWL;
				break;
			case 3: //CWU
				freq_si = freq_VFO + freq_BFO_CWU;
				break;
		}
	}
	else
	if (var_Conversion == 0)
	{
		//  - 
		switch(but_MODE)
		{
			case 0: //LSB
				freq_si = freq_VFO + freq_BFO_LSB;
				break;
			case 1: //USB
				freq_si = freq_VFO - freq_BFO_LSB;
				break;
			case 2: //CWL
				freq_si = freq_VFO + freq_BFO_CWL;
				break;
			case 3: //CWU
				freq_si = freq_VFO - freq_BFO_CWL;
				break;
		}
	}
}

void S_METR(void)											//   S-
{
	uint8_t a = 0;
		
	if (data_ADC_1 >= 8)  a = 0;	//S1
	if (data_ADC_1 >= 14) a = 1;	//S3
	if (data_ADC_1 >= 20) a = 2;	//S5
	if (data_ADC_1 >= 26) a = 3;	//S7
	if (data_ADC_1 >= 32) a = 4;	//S8
	if (data_ADC_1 >= 38) a = 5;	//S9
	if (data_ADC_1 >= 44) a = 6;	//S+20
	if (data_ADC_1 >= 50) a = 7;	//S+40
	
	if (old_S_metr_Value > a)
	{
		for(int i=8+old_S_metr_Value;i<16;i++)
		{
			LCD_set_pos(i,1);
			LCD_send_str(" ");
		}
	}
		
	old_S_metr_Value = a;
	LCD_set_pos(8,1);
	for(int b=0;b<a+1;b++)
	{
		if (b < 7)
		{
			LCD_send_char(b);
		}
		else	
			LCD_send_char(0xFF);	//S+20
	}
}

void SR_Write(char data)									//    
{
	//  
	ABCD_Code = data;
	
	unsigned char i;
	for(i=0;i<8;i++)
	{
		//
		if(data & 0x80)
			PORTD |= (1<<SR_DATA);
		else
			PORTD &= ~(1<<SR_DATA);
		
		//
		asm("nop");
		PORTD |= (1<<SR_CLOCK);
		asm("nop");
		PORTD &= ~(1<<SR_CLOCK);
		data <<= 1;
	}
	
	//
	asm("nop");
	PORTD |= (1<<SR_LATCH);
	asm("nop");
	PORTD &= ~(1<<SR_LATCH);
}

void PTT_ON(int Value)										///   TX
{
	if (Value == 1)
	{
		func_Mode = FUNC_MODE_TX;
		//      VFO/BFO_xxx  
		if (var_Swap_F_TX == 1)
		{
			if (var_Conversion == 0) //  
			{
				//  SCL1
				SCL_1_line_Control(1);
				//  VFO  si5351  BFO
				si5351aSetFrequency(freq_si);
				//  SCL1
				SCL_1_line_Control(0);
				//  BFO  si5351  VFO
				if ((but_MODE == 0)||(but_MODE == 1)) {si5351aSetFrequency(freq_BFO_LSB);}
				else
				if ((but_MODE == 2)||(but_MODE == 3)) {si5351aSetFrequency(freq_BFO_CWL);}
			}
			else
			if (var_Conversion == 1) //   
			{
				//  SCL1
				SCL_1_line_Control(1);
				//  VFO  si5351  BFO
				si5351aSetFrequency(freq_si);
				//  SCL1
				SCL_1_line_Control(0);
				//  BFO  si5351  VFO
				if (but_MODE == 0) {si5351aSetFrequency(freq_BFO_LSB);}
				else
				if (but_MODE == 1) {si5351aSetFrequency(freq_BFO_USB);}
				else
				if (but_MODE == 2) {si5351aSetFrequency(freq_BFO_CWL);}
				else
				if (but_MODE == 3) {si5351aSetFrequency(freq_BFO_CWU);}
			}
		}
		/*7     1*/
		SR_Write(ABCD_Code|1<<7);
		LCD_set_pos(0,0);
		LCD_send_str("TX-ON");
		LCD_set_pos(4,1);
		LCD_send_str("            ");
	}
	else if (Value == 0)
	{
		//      VFO/BFO_xxx  
		if (var_Swap_F_TX == 1)
		{
			if (var_Conversion == 0) //  
			{
				//  SCL1
				SCL_1_line_Control(1);
				//  BFO  si5351  BFO
				if ((but_MODE == 0)||(but_MODE == 1)) {si5351aSetFrequency(freq_BFO_LSB);}
				else
				if ((but_MODE == 2)||(but_MODE == 3)) {si5351aSetFrequency(freq_BFO_CWL);}
				//  SCL1
				SCL_1_line_Control(0);
				//  VFO  si5351  VFO
				si5351aSetFrequency(freq_si);
			}
			else
			if (var_Conversion == 1) //   
			{
				//  SCL1
				SCL_1_line_Control(1);
				//  BFO  si5351  BFO
				if (but_MODE == 0) {si5351aSetFrequency(freq_BFO_LSB);}
				else
				if (but_MODE == 1) {si5351aSetFrequency(freq_BFO_USB);}
				else
				if (but_MODE == 2) {si5351aSetFrequency(freq_BFO_CWL);}
				else
				if (but_MODE == 3) {si5351aSetFrequency(freq_BFO_CWU);}
				//  SCL1
				SCL_1_line_Control(0);
				//  VFO  si5351  VFO
				si5351aSetFrequency(freq_si);
			}
		}
		/*7     0*/
		SR_Write(ABCD_Code&~(1<<7));
		func_Mode = FUNC_MODE_NORMAL;
		Band_Label_to_LCD();
	}
}

void ATT_PRE(void)											//   ATT  PRE
{
	LCD_set_pos(4,1);
	LCD_send_str("   ");
	LCD_set_pos(4,1);
	
	switch (but_ATT_PRE)
	{
		case 0:
			SR_Write(ABCD_Code&~(1<<4));
			SR_Write(ABCD_Code&~(1<<5));
			att_pre[band_Num - 1] = 0;
			break;
		case 1:
			//4     1
			SR_Write(ABCD_Code|1<<4);
			att_pre[band_Num - 1] = 1;
			LCD_send_str("ATT");
			break;
		case 2:
			//4     0
			SR_Write(ABCD_Code&~(1<<4));
			//5     1
			SR_Write(ABCD_Code|1<<5);
			att_pre[band_Num - 1] = 2;
			LCD_send_str("PRE");
			break;
	}
	
	//  USART  ATT_PRE
	CAT_ATT_PRE_Send(0);
	
	//  ATT/PRE    EEPROM
	eeprom_write_byte((uint8_t*)48 + band_Num - 1, att_pre[band_Num - 1]);
}

void Set_MODE_L(void)										// LSB/CWL
{
	//  - 
	//  SCL1
	SCL_1_line_Control(1);
	_delay_ms(2);
	if (but_MODE == 0) {si5351aSetFrequency(freq_BFO_LSB);}
	else if (but_MODE == 2) {si5351aSetFrequency(freq_BFO_CWL);}
	_delay_ms(2);
	//  SCL1
	SCL_1_line_Control(0);
}									

void Set_MODE_H(void)										// USB/CWU
{
	//  SCL1
	SCL_1_line_Control(1);
	_delay_ms(2);
	if (var_Conversion == 1)
	{
		//  -  
		if (but_MODE == 1) {si5351aSetFrequency(freq_BFO_USB);}
		else
		if (but_MODE == 3) {si5351aSetFrequency(freq_BFO_CWU);}
	}
	else
	if (var_Conversion == 0)
	{
		//  - 
		if (but_MODE == 1) {si5351aSetFrequency(freq_BFO_LSB);}
		else
		if (but_MODE == 3) {si5351aSetFrequency(freq_BFO_CWL);}
	}
	_delay_ms(2);
	//  SCL1
	SCL_1_line_Control(0);
}									

void Check_Conversion(void)									////     
{
	//         BFO_xxx,    
	if ((var_Conversion == 0)&&(((freq_VFO <= freq_BFO_LSB)&&(but_MODE == 1))||((freq_VFO <= freq_BFO_CWL)&&(but_MODE == 3))))
	{
		but_MODE++;
		if (but_MODE > 3) but_MODE = 0;
		// 
		Set_MODE();
	}
	else
	{
		// 
		Set_MODE();	
	}
}

void Set_MODE(void)											// 
{
	LCD_set_pos(0,1);
		
	switch (but_MODE)
	{
		case 0: //LSB
			//6     0 - CW off
			SR_Write(ABCD_Code&~(1<<6));
			Set_MODE_L();
			LCD_send_str("LSB");
			break;
		case 1: //USB
			//6     0 - CW off
			SR_Write(ABCD_Code&~(1<<6));
			Set_MODE_H();
			LCD_send_str("USB");
			break;
		case 2: //CWL
			//6     1 - CW on
			SR_Write(ABCD_Code|1<<6);
			Set_MODE_L();
			LCD_send_str("CWL");
			break;
		case 3: //CWU
			//6     1 - CW on
			SR_Write(ABCD_Code|1<<6);
			Set_MODE_H();
			LCD_send_str("CWU");
		break;
	}
	//       VFO
	Calc_Out_Freq_Si5351();
	si5351aSetFrequency(freq_si);
}

void Band_Label_to_LCD(void)								//   LCD
{
	/*    LCD*/
	LCD_set_pos(0,0);
	LCD_send_str("     ");
	LCD_set_pos(0,0);
	
	LCD_send_str(band_Label[band_Num-1]);
	
	/*   */
	but_ATT_PRE = att_pre[band_Num - 1];
	ATT_PRE();
}

void Freq_to_LCD(int d)										//    LCD
{
	unsigned char n = 0;
	
	if (d == 5) // MHz
	{
		for(n=0;n<=2;n++)
			LCD_send_char(TempStr[n]);
		LCD_send_char(46);
		for(n=3;n<=5;n++)
			LCD_send_char(TempStr[n]);
	}
	
	if (d == 1) // MHz
	{
		LCD_send_char(TempStr[0]);
		LCD_send_char(39);
		for(n=1;n<=3;n++)
			LCD_send_char(TempStr[n]);
		LCD_send_char(46);
		for(n=4;n<=6;n++)
			LCD_send_char(TempStr[n]);
	}
	
	if (d == 10) // MHz
	{
		LCD_send_char(TempStr[0]);
		LCD_send_char(TempStr[1]);
		LCD_send_char(39);
		for(n=2;n<=4;n++)
			LCD_send_char(TempStr[n]);
		LCD_send_char(46);
		for(n=5;n<=7;n++)
			LCD_send_char(TempStr[n]);
	}
}

void Freq_format_to_LCD(uint32_t freq, int col, int str)	//      LCD
{
	/*    */
	ultoa(freq,TempStr,10);
	//  LCD
	//    9 999 999 Hz,   
	LCD_set_pos(col,str);
	if (freq <= 9999999)
	{
		if (freq <= 999999)
		{
			LCD_send_str("   ");
			Freq_to_LCD(5);
		}
		else
		{
			LCD_send_str(" ");
			Freq_to_LCD(1);
		}
	}
	else
	Freq_to_LCD(10);
}

void Step_Value_to_LCD(void)								//     LCD
{
	LCD_set_pos(0,1);
	LCD_send_str("        ");
	LCD_set_pos(0,1);
	if (set_Step < 1000)
	{
		itoa(set_Step,TempStr,10);
		LCD_send_str(TempStr);
		LCD_send_str(" Hz");
	}
	else
	{
		itoa(set_Step/1000,TempStr,10);
		LCD_send_str(TempStr);
		LCD_send_str(" kHz");
	}
}

void Change_STEP(int value)									//   
{
	_delay_ms(10);
	for (int a=0;a<=6;a++)
	{
		if (set_Step == step_Value[a])
			set_Step_index = a;
	}
	
	if (value == 1)
	{
		set_Step_index++;
		if (set_Step_index > 6)
			set_Step_index = 6;
		set_Step = step_Value[set_Step_index];
	}
	else
	if (value == 2)
	{
		if (set_Step_index > 0)
			set_Step_index--;
		set_Step = step_Value[set_Step_index];
	}
	
	Step_Value_to_LCD();
}

void Setup_info_to_LCD(char infoStr[16])					//    LCD   
{
	LCD_send_byte(0x01,0);
	_delay_ms(5);
	LCD_set_pos(0,0);
	LCD_send_str(infoStr);
}

uint32_t Check_Bound_Change(uint32_t value)					//   
{
	if (value <= FREQ_VFO_MIN) return value + set_Step; //FREQ_VFO_MIN;
	else
	if (value >= FREQ_VFO_MAX) return value - set_Step; //FREQ_VFO_MAX;
	else
	return value;
}

void Set_BFO(int value)										//  BFO
{
	if (func_Mode == FUNC_MODE_SET_BFO_LSB)
	{
		if (value == 1) freq_BFO_LSB = freq_BFO_LSB + set_Step;
		if (value == 2)	freq_BFO_LSB = freq_BFO_LSB - set_Step;
		
		/*      */
		freq_BFO_LSB = Check_Bound_Change(freq_BFO_LSB);
		
		//  LCD
		Freq_format_to_LCD(freq_BFO_LSB,6,1);
	}
	else
	if (func_Mode == FUNC_MODE_SET_BFO_USB)
	{
		if (value == 1) freq_BFO_USB = freq_BFO_USB + set_Step;
		if (value == 2)	freq_BFO_USB = freq_BFO_USB - set_Step;
		
		/*      */
		freq_BFO_USB = Check_Bound_Change(freq_BFO_USB);
		
		//  LCD
		Freq_format_to_LCD(freq_BFO_USB,6,1);
	}
	else
	if (func_Mode == FUNC_MODE_SET_BFO_CWL)
	{
		if (value == 1) freq_BFO_CWL = freq_BFO_CWL + set_Step;
		if (value == 2)	freq_BFO_CWL = freq_BFO_CWL - set_Step;
		
		/*      */
		freq_BFO_CWL = Check_Bound_Change(freq_BFO_CWL);
		
		//  LCD
		Freq_format_to_LCD(freq_BFO_CWL,6,1);
	}
	else
	if (func_Mode == FUNC_MODE_SET_BFO_CWU)
	{
		if (value == 1) freq_BFO_CWU = freq_BFO_CWU + set_Step;
		if (value == 2)	freq_BFO_CWU = freq_BFO_CWU - set_Step;
		
		/*      */
		freq_BFO_CWU = Check_Bound_Change(freq_BFO_CWU);
		
		//  LCD
		Freq_format_to_LCD(freq_BFO_CWU,6,1);
	}
}

void Set_XTALL(int value)									//      Si5351
{
	if (func_Mode == FUNC_MODE_SET_XTALL_1) //  Si5351 VFO
	{
		if (value == 1) xtall_FREQ_1 = xtall_FREQ_1 + set_Step;
		if (value == 2) xtall_FREQ_1 = xtall_FREQ_1 - set_Step;
		
		/*      */
		xtall_FREQ_1 = Check_Bound_Change(xtall_FREQ_1);
		
		/*     Si5351  VFO      */
		si5351aSetFrequency(FREQ_XTALL_1_DEF);
		
		//  LCD
		Freq_format_to_LCD(xtall_FREQ_1,6,1);
	}
	else
	if (func_Mode == FUNC_MODE_SET_XTALL_2) //  Si5351 BFO
	{
		if (value == 1) xtall_FREQ_2 = xtall_FREQ_2 + set_Step;
		if (value == 2) xtall_FREQ_2 = xtall_FREQ_2 - set_Step;
		
		/*      */
		xtall_FREQ_2 = Check_Bound_Change(xtall_FREQ_2);
		
		/*     Si5351  BFO      */
		si5351aSetFrequency(FREQ_XTALL_2_DEF);
		
		//  LCD
		Freq_format_to_LCD(xtall_FREQ_2,6,1);
	}	
}

void Set_use_BAND(int value)								/// 
{
	LCD_set_pos(0,1);
	LCD_send_str("         ");
	LCD_set_pos(0,1);
	
	//  LCD
	switch (value)
	{
		case 0:
			LCD_send_str(band_Label[bandDataNum]);
			LCD_send_str(": off");
			break;
		case 1:
			LCD_send_str(band_Label[bandDataNum]);
			LCD_send_str(": on");
			break;
	}
	
	//     EEPROM    
	band_use[bandDataNum] =  value;
	eeprom_write_byte((uint8_t*)57 + bandDataNum, value);
}

void DrvST_to_LCD(int value)								//    CLK0 Si5351-1/2  LCD
{
	LCD_set_pos(0,1);
	switch (value)
	{
		case 0:
			LCD_send_str("2 mA");
			break;
		case 1:
			LCD_send_str("4 mA");
			break;
		case 2:
			LCD_send_str("6 mA");
			break;
		case 3:
			LCD_send_str("8 mA");
			break;
	}
	
	if (func_Mode == FUNC_MODE_SET_DRV_ST_1)
	{
		// VFO   si5351-1 VFO
		si5351aSetFrequency(freq_si);
	}
	else
	if (func_Mode == FUNC_MODE_SET_DRV_ST_2)
	{
		// BFO_LSB   si5351-2 BFO
		si5351aSetFrequency(freq_BFO_LSB);
	}
}

void Set_DRV_ST(int value)									//    CLK0 Si5351-1/2
{
	if (value == 0) // 
	{
		if (func_Mode == FUNC_MODE_SET_DRV_ST_1)
		{
			siDSC_1--;
			if (siDSC_1 < 0) siDSC_1 = 0;
			DrvST_to_LCD(siDSC_1);
		}
		else
		if (func_Mode == FUNC_MODE_SET_DRV_ST_2)
		{
			siDSC_2--;
			if (siDSC_2 < 0) siDSC_2 = 0;
			DrvST_to_LCD(siDSC_2);
		}	
	}
	else
	if (value == 1) // 
	{
		if (func_Mode == FUNC_MODE_SET_DRV_ST_1)
		{
			siDSC_1++;
			if (siDSC_1 > 3) siDSC_1 = 3;	
			DrvST_to_LCD(siDSC_1);
		}
		else
		if (func_Mode == FUNC_MODE_SET_DRV_ST_2)
		{
			siDSC_2++;
			if (siDSC_2 > 3) siDSC_2 = 3;
			DrvST_to_LCD(siDSC_2);
		}
	}
}

void Set_Conversion(int value)								//   
{
	LCD_set_pos(0,1);
	if (value == 2)
	{
		if (var_Conversion == 0) LCD_send_str("classic  ");
		else if (var_Conversion == 1) LCD_send_str("always up");
	}
	else
	{
		//   LCD
		switch (value)
		{
			case 0:
				LCD_send_str("classic  ");
				break;
			case 1:
				LCD_send_str("always up");
				break;
		}
		var_Conversion = value;	
	}	
}

void Set_Swap_F_TX(int value)								//    VFO/BFO_xxx  TX
{
	LCD_set_pos(0,1);
	if (value == 2)
	{
		if (var_Swap_F_TX == 0) LCD_send_str("off");
		else
		if (var_Swap_F_TX == 1) LCD_send_str("on");
	}
	else
	{
		//   LCD
		switch (value)
		{
			case 0:
				LCD_send_str("off");
				break;
			case 1:
				LCD_send_str("on ");
				break;
		}
		var_Swap_F_TX = value;
	}
}

void Check_BAND(void)										//  
{
	int old_band_num = band_Num;
	int att_pre_pin_status = 0x00;
	int cw_pin_status = 0x00;
	
	//   ATT/PRE
	switch (but_ATT_PRE)
	{
		case 1:
			att_pre_pin_status = 0x10;
			break;
		case 2:
			att_pre_pin_status = 0x20;
			break;
	}
	
	//   CW
	if ((but_MODE == 2)||(but_MODE == 3)) cw_pin_status = 0x40;
			
	/* ABCD    9-   (   Q0...Q3)
	160m	= 0000
	80m		= 1000
	40m		= 0100
	30m		= 1100
	20m		= 0010
	17m		= 1010
	15m		= 0110
	12m		= 1110
	10m		= 0001
	  DCBA     +   ATT/PRE*/
	if ((freq_VFO >= FREQ_VFO_MIN)&&(freq_VFO <= BAND_1_MAX))	{band_Num = 1; SR_Write(att_pre_pin_status|cw_pin_status|0x00);} //0000
	if ((freq_VFO > BAND_1_MAX)&&(freq_VFO <= BAND_2_MAX))		{band_Num = 2; SR_Write(att_pre_pin_status|cw_pin_status|0x01);} //0001
	if ((freq_VFO > BAND_2_MAX)&&(freq_VFO <= BAND_3_MAX))		{band_Num = 3; SR_Write(att_pre_pin_status|cw_pin_status|0x02);} //0010
	if ((freq_VFO > BAND_3_MAX)&&(freq_VFO <= BAND_4_MAX))		{band_Num = 4; SR_Write(att_pre_pin_status|cw_pin_status|0x03);} //0011
	if ((freq_VFO > BAND_4_MAX)&&(freq_VFO <= BAND_5_MAX))		{band_Num = 5; SR_Write(att_pre_pin_status|cw_pin_status|0x04);} //0100
	if ((freq_VFO > BAND_5_MAX)&&(freq_VFO <= BAND_6_MAX))		{band_Num = 6; SR_Write(att_pre_pin_status|cw_pin_status|0x05);} //0101
	if ((freq_VFO > BAND_6_MAX)&&(freq_VFO <= BAND_7_MAX))		{band_Num = 7; SR_Write(att_pre_pin_status|cw_pin_status|0x06);} //0110
	if ((freq_VFO > BAND_7_MAX)&&(freq_VFO <= BAND_8_MAX))		{band_Num = 8; SR_Write(att_pre_pin_status|cw_pin_status|0x07);} //0111
	if ((freq_VFO > BAND_8_MAX)&&(freq_VFO <= FREQ_VFO_MAX))	{band_Num = 9; SR_Write(att_pre_pin_status|cw_pin_status|0x08);} //1000
	
	/*   ,  */
	if (old_band_num != band_Num)
	{
		/*     EEPROM*/
		eeprom_write_byte((uint8_t*)0x03,band_Num);
		Band_Label_to_LCD();
	}
	
	Calc_Out_Freq_Si5351();
	si5351aSetFrequency(freq_si);
}

void Change_VFO(int value)									//  
{
	//   VFO
	uint32_t oldFreqVFO = freq_VFO;
	
	if (value == 1) freq_VFO = freq_VFO + set_Step;
	if (value == 2) freq_VFO = freq_VFO - set_Step;
	
	if ((freq_VFO >= FREQ_VFO_MIN)&&(freq_VFO <= FREQ_VFO_MAX))
	{
		Check_BAND();
		
		//  LCD
		Freq_format_to_LCD(freq_VFO,6,0);
		
		timerCountSaveBandData = 0;	
	}
	else
		freq_VFO = oldFreqVFO;
}

void Set_BAND(void)											// 
{
	freq_VFO = band_last_F_BUFF[band_Num - 1];
	//     LSB    8
	if (freq_VFO <= BAND_3_MAX)
		but_MODE = 0;
	else
		but_MODE = 1;
	Set_MODE();
	Change_VFO(0);
	Band_Label_to_LCD();
}

void BandDataSaved(void)									//   
{
	//     
		
	band_last_F_BUFF[band_Num - 1] = freq_VFO;
	
	//       EEPROM
	eeprom_write_float((float*)band_last_F_data_addr[band_Num-1],band_last_F_BUFF[band_Num - 1]);
}

void Button_PAD(void)										// 
{
	if (butBuf == BUT_NULL)
	{
		/*   FUNC*/
		if ((func_Mode == FUNC_MODE_NORMAL)||(func_Mode == FUNC_MODE_SET_STEP)||(func_Mode == FUNC_MODE_SET_BFO_LSB)||(func_Mode == FUNC_MODE_SET_BFO_USB)||(func_Mode == FUNC_MODE_SET_BFO_CWL)||(func_Mode == FUNC_MODE_SET_BFO_CWU)||(func_Mode == FUNC_MODE_SET_XTALL_1)||(func_Mode == FUNC_MODE_SET_XTALL_2)||(func_Mode == FUNC_MODE_SET_BAND)||(func_Mode == FUNC_MODE_SET_DRV_ST_1)||(func_Mode == FUNC_MODE_SET_DRV_ST_2)||(func_Mode == FUNC_MODE_SET_CONV)||(func_Mode == FUNC_MODE_SET_SWP_F_TX))
		{
			/* :       */
			{
				if ((butPressFUNC == 1)&&(but_Count_Pressed > 100)&&(but_Count_Pressed < 700)&&(func_Mode == FUNC_MODE_NORMAL)&&(modeSetup == 0))
				{
					band_Num++;
					//,     ,  ,  
					while (band_use[band_Num - 1] != 1)
					{
						band_Num++;
						if (band_Num > 9) band_Num = 1;
						if (band_Num < 1) band_Num = 9;
					}
					//     EEPROM
					eeprom_write_byte((uint8_t*)0x03,band_Num);
					//   
					Set_BAND();
				}	
			}
			
			/* , 1...2 :      */
			{
				if ((butPressFUNC == 1)&&(but_Count_Pressed > 3000)&&(but_Count_Pressed < 7000)&&((func_Mode == FUNC_MODE_NORMAL)||(func_Mode == FUNC_MODE_SET_BFO_LSB)||(func_Mode == FUNC_MODE_SET_BFO_USB)||(func_Mode == FUNC_MODE_SET_BFO_CWL)||(func_Mode == FUNC_MODE_SET_BFO_CWU)||(func_Mode == FUNC_MODE_SET_XTALL_1)||(func_Mode == FUNC_MODE_SET_XTALL_2)))
				{
					modeSetup = 1;
					if (func_Mode == FUNC_MODE_NORMAL)				set_Step_F = 1;
					else if (func_Mode == FUNC_MODE_SET_BFO_LSB)	set_Step_F = 2;
					else if (func_Mode == FUNC_MODE_SET_BFO_USB)	set_Step_F = 3;
					else if (func_Mode == FUNC_MODE_SET_BFO_CWL)	set_Step_F = 4;
					else if (func_Mode == FUNC_MODE_SET_BFO_CWU)	set_Step_F = 5;
					else if (func_Mode == FUNC_MODE_SET_XTALL_1)	set_Step_F = 6;
					else if (func_Mode == FUNC_MODE_SET_XTALL_2)	set_Step_F = 7;
					
					//  
					func_Mode = FUNC_MODE_SET_STEP;
					//   LCD
					Setup_info_to_LCD("Select step");
					Step_Value_to_LCD();
				}	
			}
			
			/*      :         */
			{
				if ((butPressFUNC == 1)&&(but_Count_Pressed > 100)&&(but_Count_Pressed < 700)&&(set_Step_F != 0)&&((func_Mode == FUNC_MODE_SET_STEP)||(func_Mode == FUNC_MODE_SET_BFO_LSB)||(func_Mode == FUNC_MODE_SET_BFO_USB)||(func_Mode == FUNC_MODE_SET_BFO_CWL)||(func_Mode == FUNC_MODE_SET_BFO_CWU)||(func_Mode == FUNC_MODE_SET_XTALL_1)||(func_Mode == FUNC_MODE_SET_XTALL_2)))
				{
					//  EEPROM        step_Value
					eeprom_write_byte((uint8_t*)0x02, set_Step_index);
			
					/*    */
					if (set_Step_F == 1)
					{
						//  
						func_Mode = FUNC_MODE_NORMAL;
						LCD_send_byte(0x01,0);
						_delay_ms(5);
						Change_VFO(0);
						Band_Label_to_LCD();
						Set_MODE();
						modeSetup = 0;
					}
					else
					/*    BFO_LSB*/
					if (set_Step_F == 2)
					{
						//   -  BFO_LSB
						func_Mode = FUNC_MODE_SET_BFO_LSB;
						Setup_info_to_LCD("Freq BFO-LSB");
						//  LCD
						Freq_format_to_LCD(freq_BFO_LSB,6,1);
						modeSetup = 2;
					}
					/*    BFO_USB*/
					if (set_Step_F == 3)
					{
						//   -  BFO_USB
						func_Mode = FUNC_MODE_SET_BFO_USB;
						Setup_info_to_LCD("Freq BFO-USB");
						//  LCD
						Freq_format_to_LCD(freq_BFO_USB,6,1);
						modeSetup = 3;
					}
					else
					/*    BFO_CWL*/
					if (set_Step_F == 4)
					{
						//   -  BFO_CWL
						func_Mode = FUNC_MODE_SET_BFO_CWL;
						Setup_info_to_LCD("Freq BFO-CWL");
						//  LCD
						Freq_format_to_LCD(freq_BFO_CWL,6,1);
						modeSetup = 4;
					}
					/*    BFO_CWU*/
					if (set_Step_F == 5)
					{
						//   -  BFO_CWU
						func_Mode = FUNC_MODE_SET_BFO_CWU;
						Setup_info_to_LCD("Freq BFO-CWU");
						//  LCD
						Freq_format_to_LCD(freq_BFO_CWU,6,1);
						modeSetup = 5;
					}
					else
					/*    XTALL_1*/
					if (set_Step_F == 6)
					{
						//   -  XTALL_1
						func_Mode = FUNC_MODE_SET_XTALL_1;
						Setup_info_to_LCD("Freq XTALL-1");
						//  LCD
						Freq_format_to_LCD(xtall_FREQ_1,6,1);
						modeSetup = 6;
					}
					/*    XTALL_2*/
					if (set_Step_F == 7)
					{
						//   -  XTALL_2
						func_Mode = FUNC_MODE_SET_XTALL_2;
						Setup_info_to_LCD("Freq XTALL-2");
						//  LCD
						Freq_format_to_LCD(xtall_FREQ_2,6,1);
						modeSetup = 7;
					}
					set_Step_F = 0;
					but_Count_Pressed = 0;
				}
			}
			
			/*,  5 ,     :      BFO_L*/
			{
				if ((butPressFUNC == 1)&&(but_Count_Pressed > 14000)&&(func_Mode == FUNC_MODE_NORMAL))
				{
					//   -   BFO_LSB
					func_Mode = FUNC_MODE_SET_BFO_LSB;
					Setup_info_to_LCD("Freq BFO-LSB");
					//  LCD
					Freq_format_to_LCD(freq_BFO_LSB,6,1);
					modeSetup = 2;
				}
			}
			
			/*    */
			{
				if ((butPressFUNC == 1)&&(but_Count_Pressed > 100)&&(but_Count_Pressed < 700)&&(set_Step_F == 0)&&((modeSetup >= 2)&&(modeSetup <= 12)))
				{
					if (func_Mode == FUNC_MODE_SET_BFO_LSB)				//     BFO_USB
					{	
						//  BFO_L  EEPROM
						eeprom_write_float((float*)4,freq_BFO_LSB);
						//   -   BFO_USB
						func_Mode = FUNC_MODE_SET_BFO_USB;
						Setup_info_to_LCD("Freq BFO-USB");
						//  LCD
						Freq_format_to_LCD(freq_BFO_USB,6,1);
						modeSetup = 3;
					}
					else
					if (func_Mode == FUNC_MODE_SET_BFO_USB)				//     BFO_CWL
					{
						//  BFO_USB  EEPROM
						eeprom_write_float((float*)67,freq_BFO_USB);
						//   -   BFO_USB
						func_Mode = FUNC_MODE_SET_BFO_CWL;
						Setup_info_to_LCD("Freq BFO-CWL");
						//  LCD
						Freq_format_to_LCD(freq_BFO_CWL,6,1);
						modeSetup = 4;
					}
					else
					if (func_Mode == FUNC_MODE_SET_BFO_CWL)				//     BFO_CWU
					{
						//  BFO_CWL  EEPROM
						eeprom_write_float((float*)76,freq_BFO_CWL);
						//   -   BFO_USB
						func_Mode = FUNC_MODE_SET_BFO_CWU;
						Setup_info_to_LCD("Freq BFO-CWU");
						//  LCD
						Freq_format_to_LCD(freq_BFO_CWU,6,1);
						modeSetup = 5;
					}
					else
					if (func_Mode == FUNC_MODE_SET_BFO_CWU)				//    XTALL_1
					{
						//  BFO_CWU  EEPROM
						eeprom_write_float((float*)80,freq_BFO_CWU);
						//   -     VFO
						func_Mode = FUNC_MODE_SET_XTALL_1;
						Setup_info_to_LCD("Freq XTALL-1");
						Set_XTALL(0);
						modeSetup = 6;
					}
					else
					if (func_Mode == FUNC_MODE_SET_XTALL_1)				//    XTALL_2
					{
						//  XTALL_1  EEPROM
						eeprom_write_float((float*)8,xtall_FREQ_1);
						//   -     BFO
						func_Mode = FUNC_MODE_SET_XTALL_2;
						//  SCL1
						SCL_1_line_Control(1);
						Setup_info_to_LCD("Freq XTALL-2");
						Set_XTALL(0);
						modeSetup = 7;
					}
					else
					if (func_Mode == FUNC_MODE_SET_XTALL_2)				//       CLK0 Si5351-1
					{	
						//   XTALL_2  EEPROM
						eeprom_write_float((float*)71,xtall_FREQ_2);
						//  BFO_LSB
						si5351aSetFrequency(freq_BFO_LSB);
						//  SCL1
						SCL_1_line_Control(0);
						// VFO   si5351-1 VFO
						si5351aSetFrequency(freq_si);
						//  
						func_Mode = FUNC_MODE_SET_DRV_ST_1;
						Setup_info_to_LCD("DrvSt Si-1");
						DrvST_to_LCD(siDSC_1);
						modeSetup = 8;
					}
					else
					if (func_Mode == FUNC_MODE_SET_DRV_ST_1)			//       CLK0 Si5351-2
					{	
						//  EEPROM   siDSC_1
						eeprom_write_byte((uint8_t*)66, siDSC_1);
						//  
						func_Mode = FUNC_MODE_SET_DRV_ST_2;
						//  SCL1
						SCL_1_line_Control(1);
						// BFO_LSB   si5351-2 BFO
						si5351aSetFrequency(freq_BFO_LSB);
						Setup_info_to_LCD("DrvSt Si-2");
						DrvST_to_LCD(siDSC_2);
						modeSetup = 9;
					}
					else
					if (func_Mode == FUNC_MODE_SET_DRV_ST_2)			//      
					{
						//  EEPROM   siDSC_2
						eeprom_write_byte((uint8_t*)75, siDSC_2);
						//  SCL1
						SCL_1_line_Control(0);
						// VFO   si5351 VFO
						si5351aSetFrequency(freq_si);
						//  
						func_Mode = FUNC_MODE_SET_CONV;
						Setup_info_to_LCD("Set conversion");
						Set_Conversion(2);
						modeSetup = 10;
					}
					else					
					if (func_Mode == FUNC_MODE_SET_CONV)				//   on/off   VFO/BFO_xxx  TX
					{	
						//  EEPROM   var_Conversion
						eeprom_write_byte((uint8_t*)84, var_Conversion);
						//  
						func_Mode = FUNC_MODE_SET_SWP_F_TX;
						Setup_info_to_LCD("Swap freq on TX");
						Set_Swap_F_TX(2);
						modeSetup = 11;						
					}
					else
					if (func_Mode == FUNC_MODE_SET_SWP_F_TX)			//   on/off 
					{
						//  EEPROM   var_Swap_F_TX
						eeprom_write_byte((uint8_t*)85, var_Swap_F_TX);
						//  
						func_Mode = FUNC_MODE_SET_BAND;
						Setup_info_to_LCD("Use bands");
						bandDataNum = 0;
						Set_use_BAND(band_use[bandDataNum]);
						modeSetup = 12;
					}
					else
					if (func_Mode == FUNC_MODE_SET_BAND)				//    
					{	
						if (bandDataNum < 8)
						{
							/*   ,
							   */
							bandDataNum++;
							Set_use_BAND(band_use[bandDataNum]);
						}
						else
						{
							//  
							func_Mode = FUNC_MODE_NORMAL;
							LCD_send_byte(0x01,0);
							band_Num = 1;
							//,     ,   - 
							while (band_use[band_Num - 1] != 1)
							{
								band_Num++;
								if (band_Num > 9) band_Num = 1;
								if (band_Num < 1) band_Num = 9;
							}
							modeSetup = 0;
							//  ,  ,      
							//     EEPROM
							eeprom_write_byte((uint8_t*)0x03,band_Num);
							Set_BAND();
							Set_MODE();
						}	
					}
				}	
			}
			butPressFUNC = 0;
		}
		
		butPress_F = 0;											//    
		if ((func_Mode == FUNC_MODE_TX)&&(ptt_cat_ON_F == 0))	// PTT    CAT - 
			PTT_ON(0);			
		but_Count_Pressed = 0;
	}
	else
	{
		/* FUNC*/
		if (butBuf == BUT_FUNC)
			butPress_F = 1;
		
		/* ATT/PRE*/
		if (butBuf == BUT_ATT_PRE)
		{
			butPress_F = 1;
			if ((func_Mode == FUNC_MODE_NORMAL)&&(but_Count_Pressed > 750)&&(but_Count_Pressed < 2000))
			{
				but_ATT_PRE++;
				if (but_ATT_PRE > 2) but_ATT_PRE = 0;
				ATT_PRE();
				but_Count_Pressed = 0;
			}
		}
		
		/* MODE*/
		if (butBuf == BUT_MODE)
		{
			butPress_F = 1;
			if ((func_Mode == FUNC_MODE_NORMAL)&&(but_Count_Pressed > 1200)&&(but_Count_Pressed < 2000))
			{
				but_MODE++;
				if (but_MODE > 3) but_MODE = 0;
				//   
				Check_Conversion();
				but_Count_Pressed = 0;
			}
		}
		
		/* PTT*/
		if (butBuf == BUT_PTT)
		{
			if (func_Mode == FUNC_MODE_TX)
			{
				butPress_F = 0;
				but_Count_Pressed = 0;
			}
			else if (func_Mode == FUNC_MODE_NORMAL)
				butPress_F = 1;
				
			if ((func_Mode == FUNC_MODE_NORMAL)&&(but_Count_Pressed > 200)&&(but_Count_Pressed < 500))
				PTT_ON(1);
		}
	}
}

//// ********************************************************************************************************/

void TIMER1_Init()											// 16-  TC1
{
	TCCR1A = 0x00;			//normal   
	TCCR1B |= (1<<CS11);    //    8
	TCNT1H = 0x00;          //    
	TCNT1L = 0x00;          //    
	OCR1AH = 0x03;          //    
	OCR1AL = 0xE8;          //    
}

void ADC_Init()												// 
{
	/*      
	 - ADMUX:   VCC,  ,  23 (ADC0)  */
	ADMUX = (0<<REFS1)|(1<<REFS0)|(1<<ADLAR)|(0<<MUX3)|(0<<MUX2)|(0<<MUX1)|(0<<MUX0);
	
	/* - ADCSRA:  ,   OFF,     , 
	     ,    ,  CLK/128*/
	ADCSRA = (1<<ADEN)|(0<<ADSC)|(0<<ADATE)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

ISR(ADC_vect)												//   
{
	unsigned char data_ADC_0;
	static unsigned char LastButtonPressed;
	
	switch (ADMUX)
	{
		case 0x60:
			/*   ADC0*/
			//   
			data_ADC_0 = ADCH;
			
			//      
			if ((data_ADC_0 > 155)&&(data_ADC_0 < 165))			//  data_ADC_0 = 160  33
				butBuf = BUT_MODE;
			else if ((data_ADC_0 > 134)&&(data_ADC_0 < 146))	//  data_ADC_0 = 141  43
				butBuf = BUT_PTT;
			else if ((data_ADC_0 > 95)&&(data_ADC_0 < 105))		//  data_ADC_0 = 102  51
				butBuf = BUT_ATT_PRE;
			else if ((data_ADC_0 < 60))							//  data_ADC_0 = 055  75
				butBuf = BUT_FUNC;
			else
				butBuf = BUT_NULL;								//  
			
			//  
			if (butBuf != BUT_NULL)
			{
				if (butBuf == LastButtonPressed)
				{
					if (compare > 100)
					{
						if (LastButtonPressed == BUT_FUNC)
							//    FUNC 
							butPressFUNC = 1;
						LastButtonPressed = butBuf;
					}
					else
						compare++;
				}
				else
				{
					LastButtonPressed = butBuf;
					compare = 0;
				}				
			}
			else
			{
				compare = 0;
				butBuf = BUT_NULL;
				LastButtonPressed = butBuf;
			}
			
			ADMUX = 0x61;
			break;
		case 0x61:
			/*   ADC1*/
			//   
			data_ADC_1 = ADCH;
			
			ADMUX = 0x60;
			break;
	}
	ADCSRA |= (1<<ADSC);
}

ISR(TIMER1_COMPA_vect)										//  1
{
	next_state = PIND & ENCODER_MASK;	//   
	if (next_state != prev_state)
	{
		switch (prev_state)
		{
			case 8:
			{
				if (next_state == 12) up_state++;
				if (next_state == 0) down_state++;
				break;
			}
			case 0:
			{
				if (next_state == 8) up_state++;
				if (next_state == 4) down_state++;
				break;
			}
			case 4:
			{
				if (next_state == 0) up_state++;
				if (next_state == 12) down_state++;
				break;
			}
			case 12:
			{
				if (next_state == 4) up_state++;
				if (next_state == 8) down_state++;
				break;
			}
			default:
			{
				break;
			}
		}
		prev_state = next_state;		//   
	}
	
		
	TCNT1H = 0x00;						// 
	TCNT1L = 0x00;						//-
	
	/**/
	/*but_Count_Pressed++;
	ultoa(timerCount,TempStr,10);
	LCD_set_pos(7,1);
	LCD_send_str(TempStr);*/
	/**/
	
	//    
	if (butPress_F == 1)
	{
		but_Count_Pressed++;
		if (butPress_F >= 20000)
			butPress_F = 0;
	}
		
	//      ~2...3 
	if (timerCountSaveBandData != -1)
	{
		timerCountSaveBandData++;
		if (timerCountSaveBandData >= 4000)
		{
			timerCountSaveBandData = -1;
			BandDataSaved();
		}
	}
}

ISR(USART_RX_vect)											//   USART_RX_vect
{
	//    
	bufRX [rxIn++] = UDR0;
	rxIn &= BUFMASKRX;
}

//  *************************************************************************************************************/
int main(void)
{
	/* */
	DDRD  = 0xFF;											// D  
	PORTD = 0x4C;											//0b01001100 - 2  3 . .   , 6 . .   
	DDRB  = 0xFF;											// B  
	PORTB = 0x00;											//    B   0b00000000
	
	ADC_Init();												// 
	LCD_ini();												// 
	TIMER1_Init();											//  TC1   
	sei();													//  
	Read_Setup_EEPROM();									//   EEPROM
	i2cInit();												// I2C
	USART_Init(MYUBRR);										// USART  CAT-
	si5351aOutputOff(0);									//   Si5351-1/2 (CLK1/2)
	
	Set_BAND();												//    
	
	//      OCR1A
	TIMSK1 |= (1<<OCIE1A);
	//   RX USART
	UCSR0B |= (1<<RXCIE0);
	//   
	ADCSRA |= (1<<ADSC);
	//    USART  CAT
	CAT_IF_Send();
	
	//  ------------------------------------------------------------------------------------------------------------/
	while (1)
	{	
		/*  /  USART*/
		handlerTXUSART();
		handlerRXUSART();	
				
		/* ,    ()     
		1   4/6/8  ( up_state/down_state)       */
		{
			if ((up_state >= 8)&&(butBuf == BUT_NULL))	// 
			{
				if (func_Mode == FUNC_MODE_NORMAL)
				{
					up_state = 0;
					Change_VFO(1);
				}
				else
				if (func_Mode == FUNC_MODE_SET_STEP)
				{
					up_state = 0;
					Change_STEP(1);
				}
				else
				if ((func_Mode == FUNC_MODE_SET_BFO_LSB)||(func_Mode == FUNC_MODE_SET_BFO_USB)||(func_Mode == FUNC_MODE_SET_BFO_CWL)||(func_Mode == FUNC_MODE_SET_BFO_CWU))
				{
					up_state = 0;
					Set_BFO(1);
				}
				else
				if ((func_Mode == FUNC_MODE_SET_XTALL_1)||(func_Mode == FUNC_MODE_SET_XTALL_2))
				{
					up_state = 0;
					Set_XTALL(1);
				}
				else
				if (func_Mode == FUNC_MODE_SET_BAND)
				{
					up_state = 0;
					Set_use_BAND(1);
				}
				else
				if ((func_Mode == FUNC_MODE_SET_DRV_ST_1)||(func_Mode == FUNC_MODE_SET_DRV_ST_2))
				{
					up_state = 0;
					Set_DRV_ST(1);
				}
				else
				if (func_Mode == FUNC_MODE_SET_CONV)
				{
					up_state = 0;
					Set_Conversion(1);
				}
				else
				if (func_Mode == FUNC_MODE_SET_SWP_F_TX)
				{
					up_state = 0;
					Set_Swap_F_TX(1);
				}
			}
			if ((down_state >= 8)&&(butBuf == BUT_NULL))	// 
			{
				if (func_Mode == FUNC_MODE_NORMAL)
				{
					down_state = 0;
					Change_VFO(2);
				}
				else
				if (func_Mode == FUNC_MODE_SET_STEP)
				{
					down_state = 0;
					Change_STEP(2);
				}
				else
				if ((func_Mode == FUNC_MODE_SET_BFO_LSB)||(func_Mode == FUNC_MODE_SET_BFO_USB)||(func_Mode == FUNC_MODE_SET_BFO_CWL)||(func_Mode == FUNC_MODE_SET_BFO_CWU))
				{
					down_state = 0;
					Set_BFO(2);
				}
				else
				if ((func_Mode == FUNC_MODE_SET_XTALL_1)||(func_Mode == FUNC_MODE_SET_XTALL_2))
				{
					down_state = 0;
					Set_XTALL(2);
				}
				else
				if (func_Mode == FUNC_MODE_SET_BAND)
				{
					down_state = 0;
					Set_use_BAND(0);
				}
				else
				if ((func_Mode == FUNC_MODE_SET_DRV_ST_1)||(func_Mode == FUNC_MODE_SET_DRV_ST_2))
				{
					down_state = 0;
					Set_DRV_ST(0);
				}
				else
				if (func_Mode == FUNC_MODE_SET_CONV)
				{
					down_state = 0;
					Set_Conversion(0);
				}
				if (func_Mode == FUNC_MODE_SET_SWP_F_TX)
				{
					down_state = 0;
					Set_Swap_F_TX(0);
				}
			}	
		}
				
		/*   */
		Button_PAD();
		
		/*    S-*/
		if ((data_ADC_1 != 255)&&(func_Mode == FUNC_MODE_NORMAL)&&(butBuf == BUT_NULL)) S_METR();
	}
}
//The END **********************************************************************************************************************/